const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;
const GdkPixbuf = imports.gi.GdkPixbuf;
const Gdk = imports.gi.Gdk;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = imports.misc.extensionUtils.getCurrentExtension();

const SCHEMA_NAME = 'org.gnome.shell.extensions.replaceActivitiesText';

function init() {
}

function buildPrefsWidget() {
    let widget = new PrefsWidget();
    widget.widget.show_all();

    return widget.widget;
}

class PrefsWidget {
    constructor() {
        this.gsettings = ExtensionUtils.getSettings(SCHEMA_NAME);

        this.widget = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            border_width: 10
        });

        this.vbox = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            margin: 20, margin_top: 0
        });
        this.vbox.set_size_request(550, 650);


        this.addBoldTextToBox("Change background", this.vbox);
        this.vbox.add(new Gtk.HSeparator({margin_bottom: 5, margin_top: 5}));
        this.vbox.add(this.addPictureUrl());
        this.vbox.add(this.addTextUrl());
        this.vbox.add(this.adjustLogo());
        this.vbox.add(this.tip());

        this.widget.add(this.vbox);
    }

    addPictureUrl() {
        let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
        let setting_label = new Gtk.Label({ label: "Logo", xalign: 0, hexpand: true });
        this.setting_entry = new Gtk.Entry({ hexpand: true, margin_start: 20 });
        
        this.resetButton = new Gtk.Button({ margin_start: 5 });
        this.resetButton.set_label("Reset to Extensions's Default Icon");
        this.resetButton.connect('clicked', ()=> { this.gsettings.set_string('icon-path', '/usr/share/icons/Adwaita/scalable/emotes/face-smile-big-symbolic.svg');
        this.setting_entry.set_text(this.gsettings.get_string('icon-path')); })

        this.setting_entry.set_text(this.gsettings.get_string('icon-path'));
        this.setting_entry.connect('changed', (entry) => { this.gsettings.set_string('icon-path', entry.get_text()); });

        this.fileChooseButton = new Gtk.Button({ margin_start: 5 });
        this.fileChooseButton.set_label("Browse");
        this.fileChooseButton.connect("clicked", this.showFileChooserDialog.bind(this));

        hbox.pack_start(setting_label, false, true, 0);
        hbox.add(this.setting_entry);
        hbox.add(this.fileChooseButton);
        hbox.add(this.resetButton);

        return hbox;
    }
    
        adjustLogo() {
        let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
        let logo_label = new Gtk.Label({ label: "Adjust Icon Size", xalign: 0, hexpand: true });
        
        this.logo_adjustment = new Gtk.Adjustment({
        upper: 2,
        'step-increment': 0.1,
        'page-increment': 0.1,
        lower: 1 });
        
        this.logo_entry = new Gtk.Scale({
        hexpand: true,
        margin_start: 20,
        visible: true,
        'can-focus': true,
        digits: 1,
        adjustment: this.logo_adjustment });
        
        this.resetIconButton = new Gtk.Button({ margin_start: 5 });
        this.resetIconButton.set_label("Reset to Extensions's Default Size");
        this.resetIconButton.connect('clicked', ()=> { this.gsettings.set_double('icon-size', 1.5);
        this.logo_entry.set_value(this.gsettings.get_double('icon-size')); })
        
        this.logo_entry.set_value(this.gsettings.get_double('icon-size'));
        this.logo_entry.connect('value-changed', (entry) => { this.gsettings.set_double('icon-size', entry.get_value()); });

        hbox.add(logo_label, false, true, 1);
        hbox.add(this.logo_entry);
        hbox.add(this.resetIconButton);

        return hbox;
    }
    
    	tip() {
    	let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
        let tip_label = new Gtk.Label({ label: "Note: Use Icon of Square Shape for Setting the Logo", xalign: 0, hexpand: true });
        
        hbox.add(tip_label);
        return hbox;
    	}
    
        addTextUrl() {
        let hbox = new Gtk.Box({ orientation: Gtk.Orientation.HORIZONTAL, margin_top: 5 });
        let setting_l = new Gtk.Label({ label: "Text", xalign: 0, hexpand: true });
        this.setting_e = new Gtk.Entry({ hexpand: true, margin_start: 20 });
        
        this.resetTextButton = new Gtk.Button({ margin_start: 5 });
        this.resetTextButton.set_label("Reset to Extensions's Default Text");
        this.resetTextButton.connect('clicked', ()=> { this.gsettings.set_string('text', 'Default');
        this.setting_e.set_text(this.gsettings.get_string('text')); })

        this.setting_e.set_text(this.gsettings.get_string('text'));
        this.setting_e.connect('changed', (entry) => { this.gsettings.set_string('text', entry.get_text()); });

        hbox.add(setting_l);
        hbox.add(this.setting_e);
        hbox.add(this.resetTextButton);

        return hbox;
    }

    showFileChooserDialog() {
        let fileChooser = new Gtk.FileChooserDialog({ title: "Select File" });
        fileChooser.set_transient_for(this.widget.get_parent().get_parent());
        fileChooser.set_default_response(1);

        let filter = new Gtk.FileFilter();
        filter.add_pixbuf_formats();
        fileChooser.filter = filter;

        fileChooser.add_button("Open", Gtk.ResponseType.ACCEPT);

        fileChooser.connect("response", (dialog, response) => {
            if (response == Gtk.ResponseType.ACCEPT) {
                let file = dialog.get_file().get_path()
                if (file.length > 0)
                    this.setting_entry.set_text(file);
                fileChooser.destroy();
            }
        });

        fileChooser.show();

    }

    addBoldTextToBox(text, box) {
        let txt = new Gtk.Label({xalign: 0, margin_top: 20});
        txt.set_markup('<b>' + text + '</b>');
        txt.set_line_wrap(true);
        box.add(txt);
    }
}
